# -*- coding: utf-8 -*-

from datetime import datetime, date
from .converter import from_date


class wending(object):

    _mónþas = [
        u'Hærfest',
        u'Mist',
        u'Forst',
        u'Snáw',
        u'Regn',
        u'Wind',
        u'Sǽd',
        u'Blóstm',
        u'Mǽdland',
        u'Ríp',
        u'Hát',
        u'Wæstm',
        u'Wending'
    ]

    _easy_mónþas = [
        u'haerfest',
        u'mist',
        u'forst',
        u'snaw',
        u'regn',
        u'wind',
        u'saed',
        u'blostm',
        u'maedland',
        u'rip',
        u'hat',
        u'waestm',
        u'wending'
    ]

    def __new__(self, gere, mónþ, dæg):
        self = object.__new__(self)

        if gere < 0:
            raise ValueError("Gere must not be less than zero.")
        elif mónþ < 0 or mónþ > 13:
            raise ValueError("{} is an invalid mónþ.")
        elif dæg < 0:
            raise ValueError("Dæg must not be less than zero.")
        elif dæg > 30:
            raise ValueError("Dæg cannot be greater than 30.")
        elif (mónþ == 13 and dæg > 6):
            raise ValueError("Dæg cannot be greater than 6 for a Wending day.")

        self.gere = gere
        self.mónþ = mónþ
        self.dæg = dæg
        return self

    @classmethod
    def today(cls):
        today = datetime.combine(date.today(), datetime.min.time())
        return from_date(today)

    @classmethod
    def from_date_string(cls, date_string):
        try:
            d, m, g = date_string.split()
        except ValueError:
            raise ValueError("{} is not a valid date string"
                             .format(date_string))

        if m.lower() in cls._easy_mónþas:
            mónþas_index = cls._easy_mónþas.index(m.lower()) + 1
        else:
            raise ValueError("{} is not a valid mónþ.".format(m))

        return cls(int(g), cls._easy_mónþas.index(m.lower()) + 1, int(d))

    def formatted(self):
        return '{0} {1} {2}'.format(self.dæg,
                                    self._mónþas[self.mónþ-1],
                                    self.gere)

    def tuple(self):
        return (self.gere, self.mónþ, self.dæg)

    def __str__(self):
        return '{0}-{1}-{2}'.format(self.gere, self.mónþ, self.dæg)

    def __eq__(self, other):
        if isinstance(other, wending):
            return self._compare(other) == 0
        return NotImplemented

    def __le__(self, other):
        if isinstance(other, wending):
            return self._compare(other) <= 0
        return NotImplemented

    def __lt__(self, other):
        if isinstance(other, wending):
            return self._compare(other) < 0
        return NotImplemented

    def __ge__(self, other):
        if isinstance(other, wending):
            return self._compare(other) >= 0
        return NotImplemented

    def __gt__(self, other):
        if isinstance(other, wending):
            return self._compare(other) > 0
        return NotImplemented

    def _compare(self, other):
        a = (self.gere, self.mónþ, self.dæg)
        b = (other.gere, other.mónþ, other.dæg)
        return 0 if a == b else 1 if a > b else -1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
from datarum import __version__

setup(
    name='datarum',
    version=__version__,
    author='Mika Naylor (Autophagy)',
    author_email='mail@autophagy.io',
    url='https://github.com/Autophagy/datarum',
    description=('Datárum is a small python library to convert Gregorian '
                 'dates to Wending, an Old English variant on the French '
                 'Republican calendar, for use in various projects.'),
    long_description=open('README.rst', encoding='utf-8').read(),
    packages=['datarum'],
    python_requires='>=3',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],
)


# -*- coding: utf-8 -*-

from datetime import datetime, date, timedelta
from . import wending


# Revolutionary calendar starts on 22nd September 1792
incept = datetime(1792, 9, 21)
DAYS_IN_YEAR = 365


def from_date(date):
    if type(date) is not datetime:
        raise ValueError('Supplied date must be of type datetime.')

    if date < incept:
        raise ValueError('The calendar begins at 1792-09-22. '
                         'You cannot convert a date earlier than this.')

    # Remove the time from the timestamp
    date = datetime.combine(date, datetime.min.time())
    diff = date.timestamp() - incept.timestamp()
    return to_wending(int(round(diff / (24*60*60))))


def to_wending(total_days):
    dat = wending.wending(1, 1, 0)
    bises = False

    day_count = 1

    while (day_count <= total_days):
        dat.dæg += 1

        if (dat.dæg > 30):
            dat.dæg = 1
            dat.mónþ += 1

        if dat.mónþ == 13:
            if romme_bises(dat.gere):
                if dat.dæg > 6:
                    dat.dæg = 1
                    dat.mónþ = 1
                    dat.gere += 1
            else:
                if dat.dæg > 5:
                    dat.dæg = 1
                    dat.mónþ = 1
                    dat.gere += 1

        day_count += 1

    return dat


def to_gregorian(wending_date):
    if not isinstance(wending_date, wending.wending):
        raise ValueError('Supplied date must be of type wending.')

    return incept + timedelta(days=days_since_incept(wending_date) + 1)


def days_since_incept(wending_date):
    year_days = 0

    for i in range(1, wending_date.gere):
        if romme_bises(i):
            year_days += DAYS_IN_YEAR + 1
        else:
            year_days += DAYS_IN_YEAR

    month_days = (wending_date.mónþ-1) * 30

    return year_days + month_days + (wending_date.dæg-1)


def romme_bises(gere):
    if gere in [3, 7, 11]:
        return True
    elif gere < 15:
        return False

    return (gere % 4 == 0 and gere % 100 != 0) or gere % 400 == 0
